@@ -1,8 +1,9 @@
 {-# LANGUAGE OverloadedStrings, GeneralizedNewtypeDeriving, RecordWildCards,
-    MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances, CPP #-}
+    MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances, CPP,
+    DeriveDataTypeable #-}
 
 module Database.Redis.Core (
-    Connection(..), connect, checkedConnect,
+    Connection(..), ConnectError(..), connect, checkedConnect,
     ConnectInfo(..), defaultConnectInfo,
     Redis(), runRedis, unRedis, reRedis,
     RedisCtx(..), MonadRedis(..),
@@ -14,12 +15,15 @@ import Prelude
 #if __GLASGOW_HASKELL__ < 710
 import Control.Applicative
 #endif
+import Control.Exception
 import Control.Monad.Reader
 import qualified Data.ByteString as B
 import Data.IORef
 import Data.Pool
 import Data.Time
+import Data.Typeable
 import Network
+import Network.TLS (ClientParams)
 
 import Database.Redis.Protocol
 import qualified Database.Redis.ProtocolPipelining as PP
@@ -170,8 +174,16 @@ data ConnectInfo = ConnInfo
     -- ^ Optional timeout until connection to Redis gets
     --   established. 'ConnectTimeoutException' gets thrown if no socket
     --   get connected in this interval of time.
+    , connectTLSParams      :: Maybe ClientParams
+    -- ^ Optional TLS parameters. TLS will be enabled if this is provided.
     } deriving Show
 
+data ConnectError = ConnectAuthError Reply
+                  | ConnectSelectError Reply
+    deriving (Eq, Show, Typeable)
+
+instance Exception ConnectError
+
 -- |Default information for connecting:
 --
 -- @
@@ -181,6 +193,8 @@ data ConnectInfo = ConnInfo
 --  connectDatabase       = 0               -- SELECT database 0
 --  connectMaxConnections = 50              -- Up to 50 connections
 --  connectMaxIdleTime    = 30              -- Keep open for 30 seconds
+--  connectTimeout        = Nothing         -- Don't add timeout logic
+--  connectTLSParams      = Nothing         -- Do not use TLS
 -- @
 --
 defaultConnectInfo :: ConnectInfo
@@ -192,6 +206,7 @@ defaultConnectInfo = ConnInfo
     , connectMaxConnections = 50
     , connectMaxIdleTime    = 30
     , connectTimeout        = Nothing
+    , connectTLSParams      = Nothing
     }
 
 -- |Constructs a 'Connection' pool to a Redis server designated by the 
@@ -205,14 +220,27 @@ connect ConnInfo{..} = Conn <$>
         let timeoutOptUs =
               round . (1000000 *) <$> connectTimeout
         conn <- PP.connect connectHost connectPort timeoutOptUs
-        runRedisInternal conn $ do
+        conn' <- case connectTLSParams of
+                   Nothing -> return conn
+                   Just tlsParams -> PP.enableTLS tlsParams conn
+        PP.beginReceiving conn'
+
+        runRedisInternal conn' $ do
             -- AUTH
             case connectAuth of
                 Nothing   -> return ()
-                Just pass -> void $ auth pass
+                Just pass -> do
+                  resp <- auth pass
+                  case resp of
+                    Left r -> liftIO $ throwIO $ ConnectAuthError r
+                    _      -> return ()
             -- SELECT
-            when (connectDatabase /= 0) (void $ select connectDatabase)
-        return conn
+            when (connectDatabase /= 0) $ do
+              resp <- select connectDatabase
+              case resp of
+                  Left r -> liftIO $ throwIO $ ConnectSelectError r
+                  _      -> return ()
+        return conn'
 
     destroy = PP.disconnect
 
